 /********************************************************************/
 /*                                                                  */
 /* Program name: EVMON                                              */
 /*                                                                  */
 /* Description: C program that acts as an event monitor             */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*   EVMON is a C program that acts as an event monitor - reads an  */
 /*   event queue and tells you if anything appears on it            */
 /*                                                                  */
 /*   It's second parameter is the queue manager name, the first is  */
 /*   the event queue name. If these are not supplied it uses the    */
 /*   defaults.                                                      */
 /*                                                                  */
 /********************************************************************/
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #ifndef min
   #define min(a,b)        (((a) < (b)) ? (a) : (b))
 #endif
 #ifdef OS2
   /******************************************************************/
   /* for beep                                                       */
   /******************************************************************/
   #define INCL_DOSPROCESS
   #include <os2.h>
 #endif
 /********************************************************************/
 /* includes for MQI                                                 */
 /********************************************************************/
 #include <cmqc.h>
 #include <cmqcfc.h>

 /********************************************************************/
 /* prototype the functions                                          */
 /********************************************************************/
 void printfmqcfst(MQCFST* pmqcfst);
 void printfmqcfin(MQCFIN* pmqcfst);
 void printfmqcfil(MQCFIL* pmqcfst);
 void printreas(MQLONG reason);

 /********************************************************************/
 /* define macros                                                    */
 /********************************************************************/
  #define PRINTREAS(param)                                           \
     case param:                                                     \
       printf("Reason = %s\n",#param);                               \
       break;

 /********************************************************************/
 /* global variable                                                  */
 /********************************************************************/
 MQCFH    *evtmsg;                    /* evtmsg message buffer       */

 /********************************************************************/
 /* the program itself                                               */
 /********************************************************************/
 int main(int argc, char **argv)
 {
   /******************************************************************/
   /* declare variables                                              */
   /******************************************************************/
   int  i;                             /* auxiliary counter          */
   /******************************************************************/
   /* Declare MQI structures needed                                  */
   /******************************************************************/
   MQOD     od = {MQOD_DEFAULT};      /* Object Descriptor           */
   MQMD     md = {MQMD_DEFAULT};      /* Message Descriptor          */
   MQGMO   gmo = {MQGMO_DEFAULT};     /* get message options         */
   /******************************************************************/
   /* note, uses defaults where it can                               */
   /******************************************************************/
   MQHCONN  Hcon;                     /* connection handle           */
   MQHOBJ   Hobj;                     /* object handle               */
   MQLONG   O_options;                /* MQOPEN options              */
   MQLONG   C_options;                /* MQCLOSE options             */
   MQLONG   CompCode;                 /* completion code             */
   MQLONG   OpenCode;                 /* MQOPEN completion code      */
   MQLONG   Reason;                   /* reason code                 */
   MQLONG   CReason;                  /* reason code for MQCONN      */
   MQLONG   buflen;                   /* buffer length               */
   MQLONG   evtmsglen;                /* message length received     */
   MQCHAR   command[1100];            /* call command string ...     */
   MQCHAR   p1[600];                  /* ApplId insert               */
   MQCHAR   p2[900];                  /* evtmsg insert               */
   MQCHAR   p3[600];                  /* Environment insert          */
   MQLONG   mytype;                   /* saved application type      */
   char     QMName[50];               /* queue manager name          */
   MQCFST  *paras;                    /* the parameters              */
   int      counter;                  /* loop counter                */
   time_t   ltime;                    /* for printing the time       */

   /******************************************************************/
   /* See whether they want help                                     */
   /******************************************************************/
   if (argc > 1 && argv[1][0] == '?')
   {
     printf("Usage: evmon [EventQueue] [QMgrName]\n" );
     exit(0);
   }

   /******************************************************************/
   /* Connect to queue manager                                       */
   /* If a queue manager name is specified then use it, otherwise    */
   /* use the default one                                            */
   /******************************************************************/
   QMName[0] = 0;                      /* default queue manager      */
   if (argc > 2)
     strcpy(QMName, argv[2]);
   MQCONN(QMName,                      /* queue manager              */
          &Hcon,                       /* connection handle          */
          &CompCode,                   /* completion code            */
          &CReason);                   /* reason code                */

   /******************************************************************/
   /* report reason and stop if it failed                            */
   /******************************************************************/
   if (CompCode == MQCC_FAILED)
   {
     printf("MQCONN ended with reason code %ld\n", CReason);
     printreas(CReason);
     exit(CReason);
   }

   /******************************************************************/
   /* Initialize object descriptor for subject queue                 */
   /* By default we monitor the SYSTEM.ADMIN.QMGR.EVENT event        */
   /* queue, but we could also monitor SYSTEM.ADMIN.CHANNEL.EVENT    */
   /* or SYSTEM.ADMIN.PERFM.EVENT if these were supplied as          */
   /* parameters                                                     */
   /******************************************************************/
   strcpy(od.ObjectName, "SYSTEM.ADMIN.QMGR.EVENT");
   if (argc > 1)
     strcpy(od.ObjectName, argv[1]);

   /******************************************************************/
   /* Open the event queue for input; exclusive or shared.  Use of   */
   /* the queue is controlled by the queue definition here           */
   /******************************************************************/
   O_options = MQOO_INPUT_AS_Q_DEF     /* open queue for input       */
         + MQOO_FAIL_IF_QUIESCING      /* but not if MQM stopping    */
         + MQOO_BROWSE;
   MQOPEN(Hcon,                        /* connection handle          */
          &od,                         /* object descriptor for queue*/
          O_options,                   /* open options               */
          &Hobj,                       /* object handle              */
          &CompCode,                   /* completion code            */
          &Reason);                    /* reason code                */

   /******************************************************************/
   /* report reason, if any; stop if failed                          */
   /******************************************************************/
   if (Reason != MQRC_NONE)
   {
     printf("MQOPEN (%s) ==> %ld\n", od.ObjectName, Reason);
     printreas(Reason);
   }

   OpenCode = CompCode;                /* keep for conditional close */

   /******************************************************************/
   /*   Get messages from the message queue                          */
   /******************************************************************/
   while (CompCode != MQCC_FAILED)
   {
     /****************************************************************/
     /* We don't know how big this message is so just get the        */
     /* descriptor first                                             */
     /* wait for new messages, and when we find one lock it so that  */
     /* we can get it when we've got the storage                     */
     /* We use convert just in case this is running as a client      */
     /****************************************************************/
     gmo.Options = MQGMO_WAIT + MQGMO_LOCK
        + MQGMO_BROWSE_FIRST + MQGMO_ACCEPT_TRUNCATED_MSG
        + MQGMO_CONVERT;

     buflen = 0;                       /* ammount of message to get  */
     /****************************************************************/
     /* we used to have this as unlimited wait but we can't do       */
     /* this because if it's done from a client and the client dies  */
     /* then the server doesn't notice and keeps the queue open      */
     /* so now we loop on a 5 second wait                            */
     /****************************************************************/
     /*gmo.WaitInterval = MQWI_UNLIMITED;   no time limit            */
     gmo.WaitInterval = 5000;          /* 5 seconds                  */

     /****************************************************************/
     /* clear selectors to get messages in sequence                  */
     /****************************************************************/
     memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
     memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

     /****************************************************************/
     /* wait for event message                                       */
     /* if one doesn't arrive then wait again                        */
     /****************************************************************/
     printf("...>\n");
     for (Reason = MQRC_NO_MSG_AVAILABLE,
          CompCode = MQCC_FAILED;
          Reason == MQRC_NO_MSG_AVAILABLE
                && CompCode == MQCC_FAILED; )
     {
       MQGET(Hcon,                     /* connection handle          */
             Hobj,                     /* object handle              */
             &md,                      /* message descriptor         */
             &gmo,                     /* get message options        */
             buflen,                   /* buffer length              */
             evtmsg,                   /* evtmsg message buffer      */
             &evtmsglen,               /* message length             */
             &CompCode,                /* completion code            */
             &Reason);                 /* reason code                */
     }

     /****************************************************************/
     /* report reason, if any                                        */
     /****************************************************************/
     if (Reason != MQRC_NONE && Reason != MQRC_TRUNCATED_MSG_ACCEPTED)
     {
       printf("MQGET ==> %ld\n", Reason);
       printreas(Reason);
     }
     else
     {
       gmo.Options = MQGMO_NO_WAIT + MQGMO_MSG_UNDER_CURSOR
                     + MQGMO_CONVERT;
       buflen = evtmsglen;             /* ammount of message to get  */
       evtmsg = malloc(buflen);
       if (evtmsg != NULL)
       {
         /************************************************************/
         /* clear selectors to get messages in sequence              */
         /************************************************************/
         memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
         memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

         /************************************************************/
         /* get the event message                                    */
         /************************************************************/
         printf("...>\n");
         MQGET(Hcon,                   /* connection handle          */
               Hobj,                   /* object handle              */
               &md,                    /* message descriptor         */
               &gmo,                   /* get message options        */
               buflen,                 /* buffer length              */
               evtmsg,                 /* evtmsg message buffer      */
               &evtmsglen,             /* message length             */
               &CompCode,              /* completion code            */
               &Reason);               /* reason code                */

         /************************************************************/
         /* report reason, if any                                    */
         /************************************************************/
         if (Reason != MQRC_NONE)
         {
           printf("MQGET ==> %ld\n", Reason);
           printreas(Reason);
         }
       }
       else
       {
         CompCode = MQCC_FAILED;
       }
     }
     /****************************************************************/
     /* . . . process each message received                          */
     /****************************************************************/
     if (CompCode != MQCC_FAILED)
     {
       /**************************************************************/
       /* announce a message                                         */
       /**************************************************************/
       #ifdef OS2
         {
           unsigned short tone;
           for (tone = 1; tone < 8000; tone = tone * 2)
           {
             DosBeep(tone,50);
           }
         }
       #else
         printf("\a\a\a\a\a\a\a");
       #endif
       time(&ltime);
       printf(ctime(&ltime));

       /**************************************************************/
       /* if the data length is wrong then something has gone        */
       /* badly wrong as we've only just checked it before we got    */
       /* the storage                                                */
       /**************************************************************/
       if (evtmsglen != buflen)
         printf("DataLength = %ld?\n", evtmsglen);
       else
       {
         /************************************************************/
         /* right let's look at the data                             */
         /************************************************************/
         if (evtmsg->Type != MQCFT_EVENT)
         {
           printf("Something's wrong this isn't an event message,"
                  " it's type is %ld\n",evtmsg->Type);
         }
         else
         {
           /**********************************************************/
           /* find out what sort of event it is (we should know      */
           /* this anyway as we're looking the queue for it, but     */
           /* we'd better check it)                                  */
           /**********************************************************/
           switch (evtmsg->Command)
           {
             case MQCMD_Q_MGR_EVENT:
               printf("Queue Manager event: ");
               break;
             case MQCMD_CHANNEL_EVENT:
               printf("Channel event: ");
               break;
             case MQCMD_PERFM_EVENT:
               printf("Performance event: ");
               break;
             default:
               printf("Unknown Event message, %ld.",
                       evtmsg->Command);
               break;
           }

           /**********************************************************/
           /* Print the completion code                              */
           /**********************************************************/
           if (evtmsg->CompCode == MQCC_OK)
             printf("CompCode(OK)\n");
           else if (evtmsg->CompCode == MQCC_WARNING)
             printf("CompCode(WARNING)\n");
           else if (evtmsg->CompCode == MQCC_FAILED)
             printf("CompCode(FAILED)\n");
           else if (evtmsg->CompCode == MQCC_UNKNOWN)
             printf("CompCode(UNKNOWN)\n");
           else
             printf("* CompCode wrong * (%ld)\n",
                       evtmsg->CompCode);

           /**********************************************************/
           /* Verify that it's the right length                      */
           /**********************************************************/
           if (evtmsg->StrucLength != MQCFH_STRUC_LENGTH)
           {
             printf("it's the wrong length, %ld\n",evtmsg->StrucLength);
           }

           /**********************************************************/
           /* Verify that it's the right version                     */
           /**********************************************************/
           if (evtmsg->Version != MQCFH_VERSION_1)
           {
             printf("it's the wrong version, %ld\n",evtmsg->Version);
           }

           /**********************************************************/
           /* Verify that it's the right sequence number             */
           /**********************************************************/
           if (evtmsg->MsgSeqNumber != 1)
           {
             printf("it's the wrong sequence number, %ld\n",
                     evtmsg->MsgSeqNumber);
           }

           /**********************************************************/
           /* Verify that it's the right control option              */
           /**********************************************************/
           if (evtmsg->Control != MQCFC_LAST)
           {
             printf("it's the wrong control option, %ld\n",
                     evtmsg->Control);
           }

           /**********************************************************/
           /* Print the reason the event occurred                    */
           /**********************************************************/
           printreas(evtmsg->Reason);

           /**********************************************************/
           /* Print the parameter count, we shouldn't really need    */
           /* this but it may help if there's a problem              */
           /**********************************************************/
           printf("parameter count is %ld\n", evtmsg->ParameterCount);

           /**********************************************************/
           /* get a pointer to the start of the parameters           */
           /**********************************************************/
           paras = (MQCFST *)(evtmsg + 1);
           counter = 1;
           while (counter <= evtmsg->ParameterCount)
           {
             /********************************************************/
             /* Go through the parameters and print out the data     */
             /* associated with each one                             */
             /********************************************************/
             switch (paras->Type)
             {
               /******************************************************/
               /* MQCFT_COMMAND and MQCFT_RESPONSE are also          */
               /* parameter types but there are no instances of      */
               /* these in the events manual                         */
               /******************************************************/
               case MQCFT_INTEGER_LIST:
                 printfmqcfil((MQCFIL*)paras);
                 paras = (MQCFST *)((char *)paras
                                      + paras->StrucLength);
                 break;
               case MQCFT_STRING_LIST:
                 break;
               case MQCFT_STRING:
                 printfmqcfst(paras);
                 paras = (MQCFST *)((char *)paras
                                      + paras->StrucLength);
                 break;
               case MQCFT_INTEGER:
                 printfmqcfin((MQCFIN*)paras);
                 paras = (MQCFST *)((char *)paras
                                      + paras->StrucLength);
                 break;
               default:
                 printf("unexpected parameter type, %ld\n",
                        paras->Type);
                 counter = evtmsg->ParameterCount;
                 break;
             }
             counter++;
           }
         }
       }
       /**************************************************************/
       /* free the storage we obtained for the message               */
       /**************************************************************/
       free(evtmsg);
     }                                 /* end process for GET        */
   }                                   /* end msg processing loop    */

   /******************************************************************/
   /* close the event queue - if it was opened                       */
   /******************************************************************/
   if (OpenCode != MQCC_FAILED)
   {
     C_options = 0;                  /* no close options             */
     MQCLOSE(Hcon,                   /* connection handle            */
            &Hobj,                   /* object handle                */
            C_options,
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */
     /****************************************************************/
     /* report reason, if the closure failed                         */
     /****************************************************************/
     if (Reason != MQRC_NONE)
     {
       printf("MQCLOSE ==> %ld\n", Reason);
       printreas(Reason);
     }
   }

   /******************************************************************/
   /* Disconnect from MQM  (unless previously connected)             */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
     MQDISC(&Hcon,                   /* connection handle            */
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */
     /****************************************************************/
     /* report reason, if the disconnect failed                      */
     /****************************************************************/
     if (Reason != MQRC_NONE)
     {
       printf("MQDISC ended with reason code %ld\n", Reason);
       printreas(Reason);
     }
   }

 /********************************************************************/
 /*                                                                  */
 /* END OF EVMON                                                     */
 /*                                                                  */
 /********************************************************************/
 }

/*********************************************************************/
/*                                                                   */
/* define some macros to print out the parameters                    */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/* this is what the macro should be written as but due to a          */
/* compiler bug we've had to write a bigger version                  */
/*********************************************************************/
#if 0
#define PRINTPARAM(param)                                          \
   case param:                                                     \
     printf("%s '%.*s'\n",#param,pmqcfst->StringLength,            \
            pmqcfst->String);                                      \
     break;
#endif

#define PRINTPARAM(param)                                          \
   case param:                                                     \
     {                                                             \
       char *p = #param;                                           \
     strncpy(thestring,pmqcfst->String,min(sizeof(thestring),      \
             pmqcfst->StringLength));                              \
     thestring[pmqcfst->StringLength] = '\0';                      \
     printf("%s %s\n",p,thestring);                           \
     }                                                        \
     break;

#define PRINTAT(param)                                             \
   case param:                                                     \
     printf("MQIA_APPL_TYPE = %s\n",#param);                       \
     break;

#define PRINTFIL(param)                                            \
   case param:                                                     \
     printf("%s ",#param);                                         \
     max =  pmqcfst->Count;                                        \
     counter = 1;                                                  \
     while (counter <= max)                                        \
     {                                                             \
       printf("%ld ",pmqcfst->Values[counter - 1]);                \
       counter++;                                                  \
     }                                                             \
     printf("\n");                                                 \
     break;

#define PRINTIACF(param)                                           \
   case param:                                                     \
     printf("%s %ld (X'%lX')\n",#param,                            \
             pmqcfst->Value,pmqcfst->Value);                       \
     break;


/*********************************************************************/
/* print a string parameter                                          */
/*********************************************************************/
void printfmqcfst(MQCFST* pmqcfst)
{
  char thestring[100];

  switch (pmqcfst->Parameter)
  {
    PRINTPARAM(MQCA_APPL_ID)
    PRINTPARAM(MQCA_BACKOUT_REQ_Q_NAME)
    PRINTPARAM(MQCA_BASE_Q_NAME)
    PRINTPARAM(MQCA_COMMAND_INPUT_Q_NAME)
    PRINTPARAM(MQCA_CREATION_DATE)
    PRINTPARAM(MQCA_CREATION_TIME)
    PRINTPARAM(MQCA_DEAD_LETTER_Q_NAME)
    PRINTPARAM(MQCA_DEF_XMIT_Q_NAME)
    PRINTPARAM(MQCA_ENV_DATA)
    PRINTPARAM(MQCA_INITIATION_Q_NAME)
    PRINTPARAM(MQCA_NAMES)
    PRINTPARAM(MQCA_PROCESS_DESC)
    PRINTPARAM(MQCA_PROCESS_NAME)
    PRINTPARAM(MQCA_Q_DESC)
    PRINTPARAM(MQCA_Q_MGR_DESC)
    PRINTPARAM(MQCA_Q_MGR_NAME)
    PRINTPARAM(MQCA_Q_NAME)
    PRINTPARAM(MQCA_REMOTE_Q_MGR_NAME)
    PRINTPARAM(MQCA_REMOTE_Q_NAME)
    PRINTPARAM(MQCA_STORAGE_CLASS)
    PRINTPARAM(MQCA_TRIGGER_DATA)
    PRINTPARAM(MQCA_USER_DATA)
    PRINTPARAM(MQCA_XMIT_Q_NAME)
    PRINTPARAM(MQCACF_FROM_Q_NAME)
    PRINTPARAM(MQCACF_TO_Q_NAME)
    PRINTPARAM(MQCACF_FROM_PROCESS_NAME)
    PRINTPARAM(MQCACF_TO_PROCESS_NAME)
    PRINTPARAM(MQCACF_FROM_CHANNEL_NAME)
    PRINTPARAM(MQCACF_TO_CHANNEL_NAME)
    PRINTPARAM(MQCACF_Q_NAMES)
    PRINTPARAM(MQCACF_PROCESS_NAMES)
    PRINTPARAM(MQCACF_ESCAPE_TEXT)
    PRINTPARAM(MQCACF_LOCAL_Q_NAMES)
    PRINTPARAM(MQCACF_MODEL_Q_NAMES)
    PRINTPARAM(MQCACF_ALIAS_Q_NAMES)
    PRINTPARAM(MQCACF_REMOTE_Q_NAMES)
    PRINTPARAM(MQCACF_SENDER_CHANNEL_NAMES)
    PRINTPARAM(MQCACF_SERVER_CHANNEL_NAMES)
    PRINTPARAM(MQCACF_REQUESTER_CHANNEL_NAMES)
    PRINTPARAM(MQCACF_RECEIVER_CHANNEL_NAMES)
    PRINTPARAM(MQCACF_OBJECT_Q_MGR_NAME)
    PRINTPARAM(MQCACF_APPL_NAME)
    PRINTPARAM(MQCACF_USER_IDENTIFIER)
    PRINTPARAM(MQCACF_AUX_ERROR_DATA_STR_1)
    PRINTPARAM(MQCACF_AUX_ERROR_DATA_STR_2)
    PRINTPARAM(MQCACF_AUX_ERROR_DATA_STR_3)
    PRINTPARAM(MQCACH_CHANNEL_NAME)
    PRINTPARAM(MQCACH_DESC)
    PRINTPARAM(MQCACH_MODE_NAME)
    PRINTPARAM(MQCACH_TP_NAME)
    PRINTPARAM(MQCACH_XMIT_Q_NAME)
    PRINTPARAM(MQCACH_CONNECTION_NAME)
    PRINTPARAM(MQCACH_MCA_NAME)
    PRINTPARAM(MQCACH_SEC_EXIT_NAME)
    PRINTPARAM(MQCACH_MSG_EXIT_NAME)
    PRINTPARAM(MQCACH_SEND_EXIT_NAME)
    PRINTPARAM(MQCACH_RCV_EXIT_NAME)
    PRINTPARAM(MQCACH_CHANNEL_NAMES)
    PRINTPARAM(MQCACH_SEC_EXIT_USER_DATA)
    PRINTPARAM(MQCACH_MSG_EXIT_USER_DATA)
    PRINTPARAM(MQCACH_SEND_EXIT_USER_DATA)
    PRINTPARAM(MQCACH_RCV_EXIT_USER_DATA)
    PRINTPARAM(MQCACH_USER_ID)
    PRINTPARAM(MQCACH_PASSWORD)
    PRINTPARAM(MQCACH_LAST_MSG_TIME)
    PRINTPARAM(MQCACH_LAST_MSG_DATE)
    PRINTPARAM(MQCACH_MCA_USER_ID)
    PRINTPARAM(MQCACH_CHANNEL_START_TIME)
    PRINTPARAM(MQCACH_CHANNEL_START_DATE)
    PRINTPARAM(MQCACH_MCA_JOB_NAME)
    PRINTPARAM(MQCACH_LAST_LUWID)
    PRINTPARAM(MQCACH_CURRENT_LUWID)
    PRINTPARAM(MQCACH_FORMAT_NAME)
    default:
      printf("Invalid parameter, %ld\n",pmqcfst->Parameter);
      break;
  }
}

/*********************************************************************/
/* print a long integer parameter                                    */
/*********************************************************************/
void printfmqcfil(MQCFIL* pmqcfst)
{
  MQLONG max;
  MQLONG counter;
  switch (pmqcfst->Parameter)
  {
    PRINTFIL(MQIACH_XMIT_PROTOCOL_TYPE)
    PRINTFIL(MQIACH_BATCH_SIZE)
    PRINTFIL(MQIACH_DISC_INTERVAL)
    PRINTFIL(MQIACH_SHORT_TIMER)
    PRINTFIL(MQIACH_SHORT_RETRY)
    PRINTFIL(MQIACH_LONG_TIMER)
    PRINTFIL(MQIACH_LONG_RETRY)
    PRINTFIL(MQIACH_PUT_AUTHORITY)
    PRINTFIL(MQIACH_SEQUENCE_NUMBER_WRAP)
    PRINTFIL(MQIACH_MAX_MSG_LENGTH)
    PRINTFIL(MQIACH_CHANNEL_TYPE)
    PRINTFIL(MQIACH_DATA_COUNT)
    PRINTFIL(MQIACH_MSG_SEQUENCE_NUMBER)
    PRINTFIL(MQIACH_DATA_CONVERSION)
    PRINTFIL(MQIACH_IN_DOUBT)
    PRINTFIL(MQIACH_MCA_TYPE)
    PRINTFIL(MQIACH_CHANNEL_INSTANCE_TYPE)
    PRINTFIL(MQIACH_CHANNEL_INSTANCE_ATTRS)
    PRINTFIL(MQIACH_CHANNEL_ERROR_DATA)
    PRINTFIL(MQIACH_CHANNEL_TABLE)
    PRINTFIL(MQIACH_CHANNEL_STATUS)
    PRINTFIL(MQIACH_INDOUBT_STATUS)
    PRINTFIL(MQIACH_LAST_SEQUENCE_NUMBER)
    PRINTFIL(MQIACH_CURRENT_MSGS)
    PRINTFIL(MQIACH_CURRENT_SEQUENCE_NUMBER)
    PRINTFIL(MQIACH_MSGS)
    PRINTFIL(MQIACH_BYTES_SENT)
    PRINTFIL(MQIACH_BYTES_RECEIVED)
    PRINTFIL(MQIACH_BATCHES)
    PRINTFIL(MQIACH_BUFFERS_SENT)
    PRINTFIL(MQIACH_BUFFERS_RECEIVED)
    PRINTFIL(MQIACH_LONG_RETRIES_LEFT)
    PRINTFIL(MQIACH_SHORT_RETRIES_LEFT)
    PRINTFIL(MQIACH_MCA_STATUS)
    PRINTFIL(MQIACH_STOP_REQUESTED)
    default:
      printf("Invalid parameter, %ld\n",pmqcfst->Parameter);
      break;
  }
}

/*********************************************************************/
/* print integer parameter                                           */
/*********************************************************************/
void printfmqcfin(MQCFIN* pmqcfst)
{
  switch (pmqcfst->Parameter)
  {
    case MQIA_APPL_TYPE:
      switch (pmqcfst->Value)
      {
        PRINTAT(MQAT_UNKNOWN)
        PRINTAT(MQAT_NO_CONTEXT)
        PRINTAT(MQAT_CICS)
        PRINTAT(MQAT_MVS)
        PRINTAT(MQAT_IMS)
        PRINTAT(MQAT_OS2)
        PRINTAT(MQAT_DOS)
        PRINTAT(MQAT_AIX)
        PRINTAT(MQAT_QMGR)
        PRINTAT(MQAT_OS400)
        PRINTAT(MQAT_WINDOWS)
        PRINTAT(MQAT_CICS_VSE)
        PRINTAT(MQAT_VMS)
        PRINTAT(MQAT_GUARDIAN)
        PRINTAT(MQAT_VOS)
        PRINTAT(MQAT_USER_FIRST)
        PRINTAT(MQAT_USER_LAST)
      }
      break;
    case MQIA_Q_TYPE:
      if (pmqcfst->Value == MQQT_ALIAS)
      {
        printf("MQIA_Q_TYPE is MQQT_ALIAS\n");
      }
      else
      {
        if (pmqcfst->Value == MQQT_REMOTE)
        {
          printf("MQIA_Q_TYPE is MQQT_REMOTE\n");
          if (evtmsg->Reason == MQRC_ALIAS_BASE_Q_TYPE_ERROR)
          {
            printf("but remote is not valid here\n");
          }
        }
        else
        {
          if (pmqcfst->Value == MQQT_LOCAL)
          {
            printf("MQIA_Q_TYPE is MQQT_LOCAL\n");
          }
          else
          printf("MQIA_Q_TYPE is wrong, %ld\n",pmqcfst->Value);
        }
      }
      break;
    PRINTIACF(MQIA_AUTHORITY_EVENT)
    PRINTIACF(MQIA_BACKOUT_THRESHOLD)
    PRINTIACF(MQIA_CODED_CHAR_SET_ID)
    PRINTIACF(MQIA_COMMAND_LEVEL)
    PRINTIACF(MQIA_CURRENT_Q_DEPTH)
    PRINTIACF(MQIA_DEF_INPUT_OPEN_OPTION)
    PRINTIACF(MQIA_DEF_PERSISTENCE)
    PRINTIACF(MQIA_DEF_PRIORITY)
    PRINTIACF(MQIA_DEFINITION_TYPE)
    PRINTIACF(MQIA_HARDEN_GET_BACKOUT)
    PRINTIACF(MQIA_HIGH_Q_DEPTH)
    PRINTIACF(MQIA_INHIBIT_EVENT)
    PRINTIACF(MQIA_INHIBIT_GET)
    PRINTIACF(MQIA_INHIBIT_PUT)
    PRINTIACF(MQIA_LOCAL_EVENT)
    PRINTIACF(MQIA_MAX_HANDLES)
    PRINTIACF(MQIA_MAX_MSG_LENGTH)
    PRINTIACF(MQIA_MAX_PRIORITY)
    PRINTIACF(MQIA_MAX_Q_DEPTH)
    PRINTIACF(MQIA_MAX_UNCOMMITTED_MSGS)
    PRINTIACF(MQIA_MSG_DELIVERY_SEQUENCE)
    PRINTIACF(MQIA_MSG_DEQ_COUNT)
    PRINTIACF(MQIA_MSG_ENQ_COUNT)
    PRINTIACF(MQIA_NAME_COUNT)
    PRINTIACF(MQIA_OPEN_INPUT_COUNT)
    PRINTIACF(MQIA_OPEN_OUTPUT_COUNT)
    PRINTIACF(MQIA_PERFORMANCE_EVENT)
    PRINTIACF(MQIA_PLATFORM)
    PRINTIACF(MQIA_Q_DEPTH_HIGH_EVENT)
    PRINTIACF(MQIA_Q_DEPTH_HIGH_LIMIT)
    PRINTIACF(MQIA_Q_DEPTH_LOW_EVENT)
    PRINTIACF(MQIA_Q_DEPTH_LOW_LIMIT)
    PRINTIACF(MQIA_Q_DEPTH_MAX_EVENT)
    PRINTIACF(MQIA_Q_SERVICE_INTERVAL)
    PRINTIACF(MQIA_Q_SERVICE_INTERVAL_EVENT)
    PRINTIACF(MQIA_REMOTE_EVENT)
    PRINTIACF(MQIA_RETENTION_INTERVAL)
    PRINTIACF(MQIA_SCOPE)
    PRINTIACF(MQIA_SHAREABILITY)
    PRINTIACF(MQIA_START_STOP_EVENT)
    PRINTIACF(MQIA_SYNCPOINT)
    PRINTIACF(MQIA_TIME_SINCE_RESET)
    PRINTIACF(MQIA_TRIGGER_CONTROL)
    PRINTIACF(MQIA_TRIGGER_DEPTH)
    PRINTIACF(MQIA_TRIGGER_INTERVAL)
    PRINTIACF(MQIA_TRIGGER_MSG_PRIORITY)
    PRINTIACF(MQIA_TRIGGER_TYPE)
    PRINTIACF(MQIA_USAGE)
    PRINTIACF(MQIACF_Q_MGR_ATTRS)
    PRINTIACF(MQIACF_Q_ATTRS)
    PRINTIACF(MQIACF_PROCESS_ATTRS)
    PRINTIACF(MQIACF_FORCE)
    PRINTIACF(MQIACF_REPLACE)
    PRINTIACF(MQIACF_PURGE)
    PRINTIACF(MQIACF_QUIESCE)
    PRINTIACF(MQIACF_ALL)
    PRINTIACF(MQIACF_PARAMETER_ID)
    PRINTIACF(MQIACF_ERROR_IDENTIFIER)
    PRINTIACF(MQIACF_SELECTOR)
    PRINTIACF(MQIACF_CHANNEL_ATTRS)
    PRINTIACF(MQIACF_ESCAPE_TYPE)
    PRINTIACF(MQIACF_ERROR_OFFSET)
    PRINTIACF(MQIACF_REASON_QUALIFIER)
    PRINTIACF(MQIACF_COMMAND)
    PRINTIACF(MQIACF_OPEN_OPTIONS)
    PRINTIACF(MQIACF_AUX_ERROR_DATA_INT_1)
    PRINTIACF(MQIACF_AUX_ERROR_DATA_INT_2)
    PRINTIACF(MQIACF_CONV_REASON_CODE)
    default :
      printf("Invalid parameter, %ld\n",pmqcfst->Parameter);
      break;
  }
}

/*********************************************************************/
/* Print the reason code                                             */
/*********************************************************************/
void printreas(MQLONG reason)
{
  switch (reason)
  {
    PRINTREAS(MQRCCF_CFH_TYPE_ERROR)
    PRINTREAS(MQRCCF_CFH_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFH_VERSION_ERROR)
    PRINTREAS(MQRCCF_CFH_MSG_SEQ_NUMBER_ERR)
    PRINTREAS(MQRCCF_CFH_CONTROL_ERROR)
    PRINTREAS(MQRCCF_CFH_PARM_COUNT_ERROR)
    PRINTREAS(MQRCCF_CFH_COMMAND_ERROR)
    PRINTREAS(MQRCCF_COMMAND_FAILED)
    PRINTREAS(MQRCCF_CFIN_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFST_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFST_STRING_LENGTH_ERR)
    PRINTREAS(MQRCCF_FORCE_VALUE_ERROR)
    PRINTREAS(MQRCCF_STRUCTURE_TYPE_ERROR)
    PRINTREAS(MQRCCF_CFIN_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_CFST_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_MSG_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFIN_DUPLICATE_PARM)
    PRINTREAS(MQRCCF_CFST_DUPLICATE_PARM)
    PRINTREAS(MQRCCF_PARM_COUNT_TOO_SMALL)
    PRINTREAS(MQRCCF_PARM_COUNT_TOO_BIG)
    PRINTREAS(MQRCCF_Q_ALREADY_IN_CELL)
    PRINTREAS(MQRCCF_Q_TYPE_ERROR)
    PRINTREAS(MQRCCF_MD_FORMAT_ERROR)
    PRINTREAS(MQRCCF_REPLACE_VALUE_ERROR)
    PRINTREAS(MQRCCF_CFIL_DUPLICATE_VALUE)
    PRINTREAS(MQRCCF_CFIL_COUNT_ERROR)
    PRINTREAS(MQRCCF_CFIL_LENGTH_ERROR)
    PRINTREAS(MQRCCF_QUIESCE_VALUE_ERROR)
    PRINTREAS(MQRCCF_MSG_SEQ_NUMBER_ERROR)
    PRINTREAS(MQRCCF_PING_DATA_COUNT_ERROR)
    PRINTREAS(MQRCCF_PING_DATA_COMPARE_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_TYPE_ERROR)
    PRINTREAS(MQRCCF_PARM_SEQUENCE_ERROR)
    PRINTREAS(MQRCCF_XMIT_PROTOCOL_TYPE_ERR)
    PRINTREAS(MQRCCF_BATCH_SIZE_ERROR)
    PRINTREAS(MQRCCF_DISC_INT_ERROR)
    PRINTREAS(MQRCCF_SHORT_RETRY_ERROR)
    PRINTREAS(MQRCCF_SHORT_TIMER_ERROR)
    PRINTREAS(MQRCCF_LONG_RETRY_ERROR)
    PRINTREAS(MQRCCF_LONG_TIMER_ERROR)
    PRINTREAS(MQRCCF_SEQ_NUMBER_WRAP_ERROR)
    PRINTREAS(MQRCCF_MAX_MSG_LENGTH_ERROR)
    PRINTREAS(MQRCCF_PUT_AUTH_ERROR)
    PRINTREAS(MQRCCF_PURGE_VALUE_ERROR)
    PRINTREAS(MQRCCF_CFIL_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_MSG_TRUNCATED)
    PRINTREAS(MQRCCF_CCSID_ERROR)
    PRINTREAS(MQRCCF_ENCODING_ERROR)
    PRINTREAS(MQRCCF_DATA_CONV_VALUE_ERROR)
    PRINTREAS(MQRCCF_INDOUBT_VALUE_ERROR)
    PRINTREAS(MQRCCF_ESCAPE_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_TABLE_ERROR)
    PRINTREAS(MQRCCF_MCA_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHL_INST_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHL_STATUS_NOT_FOUND)
    PRINTREAS(MQRCCF_OBJECT_ALREADY_EXISTS)
    PRINTREAS(MQRCCF_OBJECT_WRONG_TYPE)
    PRINTREAS(MQRCCF_LIKE_OBJECT_WRONG_TYPE)
    PRINTREAS(MQRCCF_OBJECT_OPEN)
    PRINTREAS(MQRCCF_ATTR_VALUE_ERROR)
    PRINTREAS(MQRCCF_UNKNOWN_Q_MGR)
    PRINTREAS(MQRCCF_Q_WRONG_TYPE)
    PRINTREAS(MQRCCF_OBJECT_NAME_ERROR)
    PRINTREAS(MQRCCF_ALLOCATE_FAILED)
    PRINTREAS(MQRCCF_HOST_NOT_AVAILABLE)
    PRINTREAS(MQRCCF_CONFIGURATION_ERROR)
    PRINTREAS(MQRCCF_CONNECTION_REFUSED)
    PRINTREAS(MQRCCF_ENTRY_ERROR)
    PRINTREAS(MQRCCF_SEND_FAILED)
    PRINTREAS(MQRCCF_RECEIVED_DATA_ERROR)
    PRINTREAS(MQRCCF_RECEIVE_FAILED)
    PRINTREAS(MQRCCF_CONNECTION_CLOSED)
    PRINTREAS(MQRCCF_NO_STORAGE)
    PRINTREAS(MQRCCF_NO_COMMS_MANAGER)
    PRINTREAS(MQRCCF_LISTENER_NOT_STARTED)
    PRINTREAS(MQRCCF_BIND_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_INDOUBT)
    PRINTREAS(MQRCCF_MQCONN_FAILED)
    PRINTREAS(MQRCCF_MQOPEN_FAILED)
    PRINTREAS(MQRCCF_MQGET_FAILED)
    PRINTREAS(MQRCCF_MQPUT_FAILED)
    PRINTREAS(MQRCCF_PING_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_IN_USE)
    PRINTREAS(MQRCCF_CHANNEL_NOT_FOUND)
    PRINTREAS(MQRCCF_UNKNOWN_REMOTE_CHANNEL)
    PRINTREAS(MQRCCF_REMOTE_QM_UNAVAILABLE)
    PRINTREAS(MQRCCF_REMOTE_QM_TERMINATING)
    PRINTREAS(MQRCCF_MQINQ_FAILED)
    PRINTREAS(MQRCCF_NOT_XMIT_Q)
    PRINTREAS(MQRCCF_CHANNEL_DISABLED)
    PRINTREAS(MQRCCF_USER_EXIT_NOT_AVAILABLE)
    PRINTREAS(MQRCCF_COMMIT_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_ALREADY_EXISTS)
    PRINTREAS(MQRCCF_DATA_TOO_LARGE)
    PRINTREAS(MQRCCF_CHANNEL_NAME_ERROR)
    PRINTREAS(MQRCCF_XMIT_Q_NAME_ERROR)
    PRINTREAS(MQRCCF_MCA_NAME_ERROR)
    PRINTREAS(MQRCCF_SEND_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_SEC_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_MSG_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_RCV_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_XMIT_Q_NAME_WRONG_TYPE)
    PRINTREAS(MQRCCF_MCA_NAME_WRONG_TYPE)
    PRINTREAS(MQRCCF_DISC_INT_WRONG_TYPE)
    PRINTREAS(MQRCCF_SHORT_RETRY_WRONG_TYPE)
    PRINTREAS(MQRCCF_SHORT_TIMER_WRONG_TYPE)
    PRINTREAS(MQRCCF_LONG_RETRY_WRONG_TYPE)
    PRINTREAS(MQRCCF_LONG_TIMER_WRONG_TYPE)
    PRINTREAS(MQRCCF_PUT_AUTH_WRONG_TYPE)
    PRINTREAS(MQRCCF_MISSING_CONN_NAME)
    PRINTREAS(MQRCCF_CONN_NAME_ERROR)
    PRINTREAS(MQRCCF_MQSET_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_NOT_ACTIVE)
    PRINTREAS(MQRCCF_TERMINATED_BY_SEC_EXIT)
    PRINTREAS(MQRCCF_DYNAMIC_Q_SCOPE_ERROR)
    PRINTREAS(MQRCCF_CELL_DIR_NOT_AVAILABLE)
    PRINTREAS(MQRC_NONE)
    PRINTREAS(MQRC_ALIAS_BASE_Q_TYPE_ERROR)
    PRINTREAS(MQRC_ALREADY_CONNECTED)
    PRINTREAS(MQRC_BACKED_OUT)
    PRINTREAS(MQRC_BUFFER_ERROR)
    PRINTREAS(MQRC_BUFFER_LENGTH_ERROR)
    PRINTREAS(MQRC_CHAR_ATTR_LENGTH_ERROR)
    PRINTREAS(MQRC_CHAR_ATTRS_ERROR)
    PRINTREAS(MQRC_CHAR_ATTRS_TOO_SHORT)
    PRINTREAS(MQRC_CONNECTION_BROKEN)
    PRINTREAS(MQRC_DATA_LENGTH_ERROR)
    PRINTREAS(MQRC_DYNAMIC_Q_NAME_ERROR)
    PRINTREAS(MQRC_ENVIRONMENT_ERROR)
    PRINTREAS(MQRC_EXPIRY_ERROR)
    PRINTREAS(MQRC_FEEDBACK_ERROR)
    PRINTREAS(MQRC_GET_INHIBITED)
    PRINTREAS(MQRC_HANDLE_NOT_AVAILABLE)
    PRINTREAS(MQRC_HCONN_ERROR)
    PRINTREAS(MQRC_HOBJ_ERROR)
    PRINTREAS(MQRC_INHIBIT_VALUE_ERROR)
    PRINTREAS(MQRC_INT_ATTR_COUNT_ERROR)
    PRINTREAS(MQRC_INT_ATTR_COUNT_TOO_SMALL)
    PRINTREAS(MQRC_INT_ATTRS_ARRAY_ERROR)
    PRINTREAS(MQRC_SYNCPOINT_LIMIT_REACHED)
    PRINTREAS(MQRC_MAX_CONNS_LIMIT_REACHED)
    PRINTREAS(MQRC_MD_ERROR)
    PRINTREAS(MQRC_MISSING_REPLY_TO_Q)
    PRINTREAS(MQRC_MSG_TYPE_ERROR)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_Q)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_Q_MGR)
    PRINTREAS(MQRC_NO_MSG_AVAILABLE)
    PRINTREAS(MQRC_NO_MSG_UNDER_CURSOR)
    PRINTREAS(MQRC_NOT_AUTHORIZED)
    PRINTREAS(MQRC_NOT_OPEN_FOR_BROWSE)
    PRINTREAS(MQRC_NOT_OPEN_FOR_INPUT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_INQUIRE)
    PRINTREAS(MQRC_NOT_OPEN_FOR_OUTPUT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET)
    PRINTREAS(MQRC_OBJECT_CHANGED)
    PRINTREAS(MQRC_OBJECT_IN_USE)
    PRINTREAS(MQRC_OBJECT_TYPE_ERROR)
    PRINTREAS(MQRC_OD_ERROR)
    PRINTREAS(MQRC_OPTION_NOT_VALID_FOR_TYPE)
    PRINTREAS(MQRC_OPTIONS_ERROR)
    PRINTREAS(MQRC_PERSISTENCE_ERROR)
    PRINTREAS(MQRC_PERSISTENT_NOT_ALLOWED)
    PRINTREAS(MQRC_PRIORITY_EXCEEDS_MAXIMUM)
    PRINTREAS(MQRC_PRIORITY_ERROR)
    PRINTREAS(MQRC_PUT_INHIBITED)
    PRINTREAS(MQRC_Q_DELETED)
    PRINTREAS(MQRC_Q_FULL)
    PRINTREAS(MQRC_Q_NOT_EMPTY)
    PRINTREAS(MQRC_Q_SPACE_NOT_AVAILABLE)
    PRINTREAS(MQRC_Q_TYPE_ERROR)
    PRINTREAS(MQRC_Q_MGR_NAME_ERROR)
    PRINTREAS(MQRC_Q_MGR_NOT_AVAILABLE)
    PRINTREAS(MQRC_REPORT_OPTIONS_ERROR)
    PRINTREAS(MQRC_SECOND_MARK_NOT_ALLOWED)
    PRINTREAS(MQRC_SECURITY_ERROR)
    PRINTREAS(MQRC_SELECTOR_COUNT_ERROR)
    PRINTREAS(MQRC_SELECTOR_LIMIT_EXCEEDED)
    PRINTREAS(MQRC_SELECTOR_ERROR)
    PRINTREAS(MQRC_SELECTOR_NOT_FOR_TYPE)
    PRINTREAS(MQRC_SIGNAL_OUTSTANDING)
    PRINTREAS(MQRC_SIGNAL_REQUEST_ACCEPTED)
    PRINTREAS(MQRC_STORAGE_NOT_AVAILABLE)
    PRINTREAS(MQRC_SYNCPOINT_NOT_AVAILABLE)
    PRINTREAS(MQRC_TRIGGER_CONTROL_ERROR)
    PRINTREAS(MQRC_TRIGGER_DEPTH_ERROR)
    PRINTREAS(MQRC_TRIGGER_MSG_PRIORITY_ERR)
    PRINTREAS(MQRC_TRIGGER_TYPE_ERROR)
    PRINTREAS(MQRC_TRUNCATED_MSG_ACCEPTED)
    PRINTREAS(MQRC_TRUNCATED_MSG_FAILED)
    PRINTREAS(MQRC_UNKNOWN_ALIAS_BASE_Q)
    PRINTREAS(MQRC_UNKNOWN_OBJECT_NAME)
    PRINTREAS(MQRC_UNKNOWN_OBJECT_Q_MGR)
    PRINTREAS(MQRC_UNKNOWN_REMOTE_Q_MGR)
    PRINTREAS(MQRC_WAIT_INTERVAL_ERROR)
    PRINTREAS(MQRC_XMIT_Q_TYPE_ERROR)
    PRINTREAS(MQRC_XMIT_Q_USAGE_ERROR)
    PRINTREAS(MQRC_NOT_OPEN_FOR_PASS_ALL)
    PRINTREAS(MQRC_NOT_OPEN_FOR_PASS_IDENT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET_ALL)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET_IDENT)
    PRINTREAS(MQRC_CONTEXT_HANDLE_ERROR)
    PRINTREAS(MQRC_CONTEXT_NOT_AVAILABLE)
    PRINTREAS(MQRC_SIGNAL1_ERROR)
    PRINTREAS(MQRC_OBJECT_ALREADY_EXISTS)
    PRINTREAS(MQRC_OBJECT_DAMAGED)
    PRINTREAS(MQRC_RESOURCE_PROBLEM)
    PRINTREAS(MQRC_ANOTHER_Q_MGR_CONNECTED)
    PRINTREAS(MQRC_UNKNOWN_REPORT_OPTION)
    PRINTREAS(MQRC_SUPPRESSED_BY_EXIT)
    PRINTREAS(MQRC_FORMAT_ERROR)
    PRINTREAS(MQRC_SOURCE_CCSID_ERROR)
    PRINTREAS(MQRC_SOURCE_INTEGER_ENC_ERROR)
    PRINTREAS(MQRC_SOURCE_DECIMAL_ENC_ERROR)
    PRINTREAS(MQRC_SOURCE_FLOAT_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_CCSID_ERROR)
    PRINTREAS(MQRC_TARGET_INTEGER_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_DECIMAL_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_FLOAT_ENC_ERROR)
    PRINTREAS(MQRC_NOT_CONVERTED)
    PRINTREAS(MQRC_CONVERTED_MSG_TOO_BIG)
    PRINTREAS(MQRC_ADAPTER_STORAGE_SHORTAGE)
    PRINTREAS(MQRC_ADAPTER_CONN_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_SERV_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_DEFS_ERROR)
    PRINTREAS(MQRC_ADAPTER_DEFS_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_DISC_LOAD_ERROR)
    PRINTREAS(MQRC_CICS_WAIT_FAILED)
    PRINTREAS(MQRC_SOURCE_LENGTH_ERROR)
    PRINTREAS(MQRC_TARGET_LENGTH_ERROR)
    PRINTREAS(MQRC_SOURCE_BUFFER_ERROR)
    PRINTREAS(MQRC_TARGET_BUFFER_ERROR)
    PRINTREAS(MQRC_DBCS_ERROR)
    PRINTREAS(MQRC_TRUNCATED)
    PRINTREAS(MQRC_ASID_MISMATCH)
    PRINTREAS(MQRC_CONN_ID_IN_USE)
    PRINTREAS(MQRC_Q_MGR_QUIESCING)
    PRINTREAS(MQRC_Q_MGR_STOPPING)
    PRINTREAS(MQRC_DUPLICATE_RECOV_COORD)
    PRINTREAS(MQRC_PMO_ERROR)
    PRINTREAS(MQRC_API_EXIT_NOT_FOUND)
    PRINTREAS(MQRC_API_EXIT_LOAD_ERROR)
    PRINTREAS(MQRC_REMOTE_Q_NAME_ERROR)
    PRINTREAS(MQRC_GMO_ERROR)
    PRINTREAS(MQRC_PAGESET_FULL)
    PRINTREAS(MQRC_PAGESET_ERROR)
    PRINTREAS(MQRC_NAME_NOT_VALID_FOR_TYPE)
    PRINTREAS(MQRC_UNEXPECTED_ERROR)
    PRINTREAS(MQRC_UNKNOWN_XMIT_Q)
    PRINTREAS(MQRC_UNKNOWN_DEF_XMIT_Q)
    PRINTREAS(MQRC_DEF_XMIT_Q_TYPE_ERROR)
    PRINTREAS(MQRC_DEF_XMIT_Q_USAGE_ERROR)
    PRINTREAS(MQRC_NAME_IN_USE)
    PRINTREAS(MQRC_CONNECTION_QUIESCING)
    PRINTREAS(MQRC_CONNECTION_STOPPING)
    PRINTREAS(MQRC_ADAPTER_NOT_AVAILABLE)
    PRINTREAS(MQRC_MSG_ID_ERROR)
    PRINTREAS(MQRC_CORREL_ID_ERROR)
    PRINTREAS(MQRC_FILE_SYSTEM_ERROR)
    PRINTREAS(MQRC_NO_MSG_LOCKED)
    PRINTREAS(MQRC_CONNECTION_NOT_AUTHORIZED)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_CHANNEL)
    PRINTREAS(MQRC_CALL_IN_PROGRESS)
    PRINTREAS(MQRC_Q_MGR_ACTIVE)
    PRINTREAS(MQRC_Q_MGR_NOT_ACTIVE)
    PRINTREAS(MQRC_Q_DEPTH_HIGH)
    PRINTREAS(MQRC_Q_DEPTH_LOW)
    PRINTREAS(MQRC_Q_SERVICE_INTERVAL_HIGH)
    PRINTREAS(MQRC_Q_SERVICE_INTERVAL_OK)
    PRINTREAS(MQRC_HCONFIG_ERROR)
    PRINTREAS(MQRC_FUNCTION_ERROR)
    PRINTREAS(MQRC_CHANNEL_STARTED)
    PRINTREAS(MQRC_CHANNEL_STOPPED)
    PRINTREAS(MQRC_CHANNEL_CONV_ERROR)
    PRINTREAS(MQRC_SERVICE_NOT_AVAILABLE)
    PRINTREAS(MQRC_INITIALIZATION_FAILED)
    PRINTREAS(MQRC_TERMINATION_FAILED)
    PRINTREAS(MQRC_UNKNOWN_Q_NAME)
    PRINTREAS(MQRC_SERVICE_ERROR)
    PRINTREAS(MQRC_Q_ALREADY_EXISTS)
    PRINTREAS(MQRC_USER_ID_NOT_AVAILABLE)
    PRINTREAS(MQRC_UNKNOWN_ENTITY)
    PRINTREAS(MQRC_UNKNOWN_AUTH_ENTITY)
    PRINTREAS(MQRC_UNKNOWN_REF_OBJECT)
    default:
      printf("It's an unknown reason, %ld\n",
              reason);
      break;
  }
}
